#version 450

in vec2 uv;

uniform sampler2D albedo;
uniform sampler2D depth;
uniform sampler2D ambient_occlusion;
uniform mat4 inverse_projection;
uniform vec3 sun_light;
uniform vec3 sky_color;
uniform float fog_start;
uniform float fog_end;
uniform vec3 fog_color;

out vec4 color;

// Unreal 3, Documentation: "Color Grading"
// Adapted to be close to Tonemap_ACES, with similar range
// Gamma 2.2 correction is baked in, don't use with sRGB conversion!
vec3 unreal(vec3 x) {
  return x / (x + 0.155) * 1.019;
}

vec3 calculate_view_position(float depth)
{
	vec4 viewPos = inverse_projection * vec4(uv.x * 2 - 1., uv.y * 2 - 1., 2.0 * depth - 1.0, 1.0);
	return viewPos.xyz / viewPos.w;
}

void main() {
	float depth = texture(depth, uv).x;

	gl_FragDepth = depth;

	vec4 albedo_color = texture(albedo, uv);

	// Skybox!
	if (depth == 1.) {
		color = vec4(unreal(albedo_color.rgb), 1.0);

		return;
	}

	vec3 view_pos = calculate_view_position(depth);

	// Apply Ambient Occlusion
	albedo_color.rgb = albedo_color.rgb * texture(ambient_occlusion, uv).r;

	// Apply fog
	float fog_factor = smoothstep(fog_start, fog_end, length(view_pos));
	albedo_color = mix(albedo_color, vec4(fog_color, 1.0), fog_factor);
	
	// Unreal 3 tonemapper, applies gamma as well
	color = vec4(unreal(albedo_color.rgb), 1.0);
}